#!/usr/bin/python
# E_waveform_envelope_spectrum_display.py. D. Gibbon, 2021-07-06

import sys, re
import numpy as np
import matplotlib.pyplot as plt
import scipy.io.wavfile as wave
from scipy.signal import butter, lfilter

cutoff = 5; order = 5

if len(sys.argv) > 1:
	wavfilename = sys.argv[1]					# get input filename from command line 
else:
	wavfilename = "DATA/one-to-ten-Putonghua-Lara-16k-mono.wav"
wavfilebase = re.sub("^.*/","",wavfilename)
wavfilebase = re.sub("-16k-mono","",wavfilebase[:-4])
figurefilename = "PNG/E_%s.png"%wavfilebase

fs, signal = wave.read(wavfilename)				# read sampling frequency and signal
signallength = len(signal)						# define signal length in bytes
signalseconds = signallength / fs				# define signal length in seconds
signal = signal / max(abs(signal))				# normalise signal -1 ... 0 ... 1

b, a = butter(cutoff, order / (0.5 * fs), btype="low")	# define Butterworth filter
envelope = lfilter(b, a, abs(signal))			# apply filter to create lf envelope
envelope = envelope / max(envelope)				# normalise envelope 0 ... 1

specmags = np.abs(np.fft.rfft(envelope))		# spectrum magnitude values
specmags = specmags / np.max(specmags)			# normalise 0...1
specmaglen = len(specmags)
specfreqs = np.linspace(0,fs/2,specmaglen)		# frequencies for each magnitude value

spectrummax = 3
topmagscount = 6								# define max frequency of lf spectrum
lfspecmaglen = int(round(spectrummax * specmaglen / (fs / 2)))
lfspecmags = specmags[1:lfspecmaglen]
lfspecfreqs = specfreqs[1:lfspecmaglen]
#-----------------------------------------------------------------------------------------
fig,((plt01, plt02)) = plt.subplots(nrows=1, ncols=2, figsize=(14, 4))	# define figure format
plt.suptitle("%s, %d"%(wavfilename, fs), fontweight="bold")	# display a title

xaxistime = np.linspace(0, signalseconds, signallength)	# define x axis in seconds
plt01.plot(xaxistime, signal, color="lightgrey")		# plot waveform in grey
plt01.plot(xaxistime, envelope, color="red")
plt01.set_xlabel("Time")
plt01.set_ylabel("Amplitude")
plt01.set_xlim(0, np.ceil(signalseconds))

plt02.plot(lfspecfreqs, lfspecmags)
plt02.set_xlabel("Frequency")
plt02.set_ylabel("Magnitude")
plt02.set_xlim(0,spectrummax)

plt.tight_layout(pad=3)
plt.savefig(figurefilename)
plt.show()												# display figure
